package org.ids_mannheim;

import java.time.Instant;
import java.time.LocalDateTime;
import java.time.ZoneId;
import java.time.format.DateTimeFormatter;

public class Progressbar {
    private final long max;
    private long current;
    private final long start;
    private long lastUpdate;
    private static final int width=40;
//    private static final DateTimeFormatter formatter = DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss");
    private static final DateTimeFormatter formatter = DateTimeFormatter.ofPattern("ccc dd. LLL HH:mm:ss");

    public Progressbar(long max, String name) {
        this.start = System.currentTimeMillis();
      this.max = max;
        System.err.println(name + ":");
        this.printBar(false);
    }

    public void setVal(long i) {
        this.current = i;
        if ((System.currentTimeMillis() - this.lastUpdate) > 100) {
            this.lastUpdate = System.currentTimeMillis();
            this.printBar(false);
        }
    }

    public void update(long i) {
        this.current += i;
        if ((System.currentTimeMillis() - this.lastUpdate) > 100) {
            this.lastUpdate = System.currentTimeMillis();
            this.printBar(false);
        }
    }

    public void finish() {
        this.current = this.max;
        this.printBar(true);
    }

    public String shortDateTimeFromMillis(long millis) {
        LocalDateTime date = LocalDateTime.ofInstant(Instant.ofEpochMilli(millis), ZoneId.systemDefault());
        return date.format(formatter);
    }

    public String hmsFromMillis(long millis) {
        int seconds = (int) (millis / 1000) % 60;
        int minutes = (int) (millis / 1000) % 3600 / 60;
        int hours = (int) (millis / 1000) / 3600;
        return String.format("%02d:%02d:%02d", hours, minutes, seconds);
    }

    private void printBar(boolean finished) {
        double numbar = Math.floor(width * (double) current / (double) max);
        StringBuilder strbar = new StringBuilder();
        String percent = String.format("%3d%%", (int)(100 * (double) current / (double) max));
        int i;
        for (i = 0; i < numbar-1; i++) {
            strbar.append("=");
        }
        for (i = (int) numbar-1; i < numbar; i++) {
          if (i < width-1) {
            strbar.append(">");
          } else {
            strbar.append("=");
          }
        }
        for (i = (int) numbar; i < width; i++) {
            strbar.append(" ");
        }
        long elapsed = (System.currentTimeMillis() - this.start);
        String strend = hmsFromMillis(elapsed);

        String strETA;
        if (elapsed < 2000) {
            strETA = "--:--:--";
        } else {
            long timeETA = elapsed * (long) ((double) max / (double) current);
            strETA = hmsFromMillis(timeETA) + " -> " + shortDateTimeFromMillis(start + elapsed + timeETA);
        }

        String speed = String.format("@%.1fMB/s", 1000.0 * current / (1024*1024) / elapsed);
        if (finished) {
            strend = "Finished: " + speed + " " + strend + " = " + shortDateTimeFromMillis(System.currentTimeMillis());
        } else {
//          strend = "Elapsed: " + strend + " ETA: " + strETA + "\u001B[0K";
            strend = speed + " ETA: " + strETA + "\u001B[0K";
        }
        System.err.print("\r" + percent+ " [" + strbar + "] " + strend + "\u001B[0K");
        if (finished) {
            System.err.print("\n");
        }
        System.err.flush();
    }
}