package org.ids_mannheim;

import java.io.BufferedReader;
import java.io.File;
import java.io.IOException;
import java.io.InputStreamReader;
import java.util.ArrayList;
import java.util.concurrent.BlockingQueue;
import java.util.concurrent.ConcurrentHashMap;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

public class Worker implements Runnable {
    private static final Pattern new_text_pattern = Pattern.compile("^#\\s+text_id\\s*=\\s*(.+)");
    private final ArrayList<String> fnames;
    private final BlockingQueue<Integer> queue;
    private final ConcurrentHashMap<String, FoldedEntry> map;
    private final int folds;
    private final Progressbar etaPrinter;
    private final int ngram_size;

    public Worker(BlockingQueue<Integer> queue, ArrayList<String> fnames, int ngram_size, int folds, ConcurrentHashMap<String, FoldedEntry> map, ETAPrinter etaPrinter) {
        this.queue = queue;
        this.fnames = fnames;
        this.map = map;
        this.ngram_size = ngram_size;
        this.folds = folds;
        this.etaPrinter = etaPrinter;
    }

    @Override
    public void run() {
        try {
            int index = queue.take();
            while (index >= 0) {
                String fname = fnames.get(index);
                long file_size = new File(fname).length();
                System.err.println(Thread.currentThread().getName() + " - processing: " + fname);
                String[] cmd = {
                        "/bin/sh",
                        "-c",
                        "/usr/local/kl/bin/korapxml2conllu " + fname
                };
                Process p = Runtime.getRuntime().exec(cmd);
                BufferedReader in = new BufferedReader(new InputStreamReader(p.getInputStream()));
                String line;
                int fold = -1;
                SlidingWindowQueue slidingWindowQueue = null;
                while ((line = in.readLine()) != null) {
                    if (line.startsWith("#")) {
                        Matcher matcher = new_text_pattern.matcher(line);
                        if (matcher.find()) {
                            fold = Math.abs(matcher.group(1).hashCode()) % folds +1;
                        }
                        int finalFold = fold;
                        slidingWindowQueue = new SlidingWindowQueue(ngram_size, s -> {
                            FoldedEntry.incr(map, s, finalFold);
                        });
                        continue;
                    }
                    String[] strings = line.split("\\s+");
                    if (strings.length < 4) {
                        continue;
                    }
                    slidingWindowQueue.add(strings[1]);
                }
                etaPrinter.update(file_size);
                index = queue.take();
            }
        } catch (InterruptedException | IOException e) {
            Thread.currentThread().interrupt();
        }
    }
}
